#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sched.h>

/*
 * This 'test' is designed to run forever.  It must manually be killed,
 * so it is not ideally suited tobe part of a validation suite of tests.
 * This test was initially designed to look for 'delays' between two
 * calls to clock_gettime(), and helped locate SMI induced delays on
 * several hardware platforms.
 *
 * As mentioned above, this test is designed to be run on a system for
 * an unspecified period of time.  It would not be unusual to let this
 * test run for several days.
 *
 * During startup, the test will print out the 'maximum' delay between
 * clock_gettime() calls.  It starts with a predefined maximum of
 * START_MAX(300ns) to eliminate noise during startup.  In addition,
 * it will print out every delay that exceeds REPORT_MIN (1000000ns).
 */

/* #define CLOCK_TO_USE CLOCK_MONOTONIC */
#define CLOCK_TO_USE CLOCK_REALTIME

#define START_MAX	3000
#define REPORT_MIN	1000000

main()
{
	int i, rc;
	struct timespec ts, p_ts;
	long long s_time, e_time, diff_time;
	long long max_time = START_MAX;
	cpu_set_t mask;
	struct sched_param param;
	time_t tt;

/*
        CPU_ZERO(&mask);
        CPU_SET(0, &mask);
        rc = sched_setaffinity(0, sizeof(mask), &mask);
        if (rc) {
		perror("sched_setaffinity");
                exit(1);
        }
*/

        param.sched_priority = sched_get_priority_min(SCHED_FIFO) + 80;
        rc = sched_setscheduler(0, SCHED_FIFO, &param);
	if (rc) {
		perror("sched_setscheduler");
		exit(1);
	}

	rc = clock_gettime(CLOCK_TO_USE, &p_ts);
	if (rc) {
		perror("clock_gettime");
		exit(1);
	}

	while(1) {
		rc = clock_gettime(CLOCK_TO_USE, &ts);
		if (rc) {
			perror("clock_gettime");
			exit(1);
		}

		s_time = p_ts.tv_sec;
		s_time *= 1000000000UL;
		s_time += p_ts.tv_nsec;

		e_time = ts.tv_sec;
		e_time *= 1000000000UL;
		e_time += ts.tv_nsec;

		diff_time = e_time - s_time;
		if ((diff_time > max_time) ||
		    (diff_time > REPORT_MIN)) {
			tt = (time_t)ts.tv_sec;
			printf("Task delayed for %lld nsec!!! %s",
				diff_time, ctime (&tt));
			fflush(stdout);
			if (diff_time > max_time)
				max_time = diff_time;

			rc = clock_gettime(CLOCK_TO_USE, &p_ts);
			if (rc) {
				perror("clock_gettime");
				exit(1);
			}
		} else {
			p_ts = ts;
		}
	}
}
